/* ***** BEGIN LICENSE BLOCK *****
 *
 * SmoothWheel: Smart algorithms for smooth scrolling with the mouse wheel.
 * Copyright (C) 2003 Avi Halachmi
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * contact:  avihpit (*at*) yahoo (*dot*) com
 *
 * The Original Code is SmoothWheel.
 *
 * The Initial Developer of the Original Code is Avi Halachmi.
 * Portions created by the Initial Developer are Copyright (C) 2003 Avi Halachmi.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */


/*********************************************************
 *
 * SmoothWheel by Avi Halachmi (avihpit (*at*) yahoo (*dot*) com)
 * http://smoothwheel.mozdev.org
 * See complete changelog and more info at the bottom of this file.
 *
 * started  (Avi Halachmi): 22-Mar-2003 (v0.1)
 *
 * modified (Avi Halachmi): 15-Apr-2003 (v0.2):
 *  - Improved Algorithm, Improved Timing, much improved target detection,
 *    Modifier keys, Better configuration and presets, Optimizations.
 *
 * modified (Avi Halachmi): 22-May-2003 (v0.3):
 *  - New target detection code (courtesy of All-In-One-Gestures author), "disable" Modifier key,
 *    soft-edge, adaptive-step, adaptive-duration, works smoothly on view-src and composer as well,
 *    namespace improvements, more event handler optimizations for the default settings, some more presets.
 *
 * modified (Avi Halachmi): 2003-06-05 (v0.31):
 *  - Compatibility with Scroll-Wheel-Navigation at All-In-One-Gestures extension fixed,
 *  - Slight code cleanups.
 *
 * modified (Avi Halachmi): 2003-08-02 (v0.32):
 *  - Works in Mozilla-Thunderbird and Help window (Mozilla/Netscape).
 *  - Improved compatibility with Mozilla-Firebird 0.6.1 and Mozilla-Thunderbird 0.1.
 *
 * modified (Avi Halachmi): 2004-01-27 (v0.33):
 *  - Improved compatibility with latest Moz abd Birds releases (detection code from AIO v0.96)
 *
 * modified (Avi Halachmi): 2004-02-01 (v0.33.1):
 *  - Removed 'handled(normal)' alert if default settings are changed.
 *
 * modified (Avi Halachmi): 2004-02-04 (v0.4):
 *  - Refined algorithms implementation (a bit more responsive)
 *  - Averaged mouse wheel events (better predictability and stability)
 *  - Fixed: occasional misbehaviour if not using a preset.
 *
 * modified (Avi Halachmi): 2004-10-22 (v0.41, 0.42):
 *  - Added compatibility with Firefox 1.0 and Thunderbird 1.0 new Extension-manager API
 *
 * modified (Avi Halachmi): 2004-11-07 (v0.43):
 *  - Added configuration panel
 *
 * modified (Avi Halachmi): 2004-11-07 (v0.43.1.20041107):
 *  - max-version reduced such that it's accepted by update.mozilla.org
 *  - added build date to the version number
 *  - added tooltip for the modifiers section in the configuration panel
 *
 * modified (Avi Halachmi): 2005-05-23 (v0.44.5.20050523):
 *  - added more variables to the configuration panel
 *  - modified prefutils a bit to allow foating point values (as strings)
 *  - more options exposed, better usability by rearrangement of settings and an about panel.
 *  - clean all smoothwheel preferences if upgrading a version
 *
 * modified (Avi Halachmi): 2005-05-27 (v0.44.6.20050527)
 *  - bugfix: on linux only with firefox 1.04 scrolling over a (scrollable) drop-down box scrolled the page instead of the box. fixed.
 *  - bugfix: on linux and windows scrolling over a non-scrollable drop-down box scrolled the page (instead of doing nothing). fixed.
 *
 * modified (Avi Halachmi): 2005-05-27 (v0.44.6.20050527)
 *  - added NV|U and netscape 8. bumped maxVersion for DeerPark Alpha
 *
 * modified (Avi Halachmi): 2007-10-26 (v0.44.10.20071026)
 *  - added an option to capture keyboard up/down as smoothwheel events
*********************************************************/


//**************** CONFIGURATION SECTION ****************

// ********** IMPORTANT ***************
//         all the settings that are affected by a preset are irrelevant now (since v0.43)
//         due to the new configuration dialog
//    Mozilla/Netscape users, enter this address: chrome://smoothwheel/content/prefoverlay.xul
///////////////////////////////////////

const sw_currentVersion="0.44.10.20071026";

//Global enable/disable for this extension. default is true.
// setting it to false will disable this extension completely.
var sw_EnableThisExtension=true;

// select one of several preset settings: (default is 7)
// 0 : don't use preset. use actual variables values.
// 1 : same as 7, but with duration=100;
// 2 : same as 7, but with duration=200;
// 3 : medium: [was the default in v0.2] (duration=500, non-linear, smooth transitions, relative step)
// 4 : nice and easy: yet even slower and smoother   (duration=1000, non-linear, smooth transitions, relative step)
// 5 : fast + adaptive-step
// 6 : fast + adaptive-duration
// 7 :  -> [DEFAULT] fast + adaptive-step + adaptive-duration
// 8 : medium + adaptive-step
// 9 : medium + adaptive-step + adattive-duration
// 10 : same as version 0.1 default (duration=250, linear, no smooth transition, no relative step)
// 11 : same as version 0.1 with isLinear=false, duration=150 (no smooth transition, no relative step)
// 12 : turbo:  for those who like quickies  (duration=150, non-linear, smooth transitions, relative step)
// 13: fast: slower than turbo, faster than the medium   (duration=300, non-linear, smooth transitions, relative step)
var sw_PresetSettings=0;

//max milisec to complete scroll after last scroll event
//reasonable values are (fast)100/250/500/1000(slow). default is 300
// -> this value is overriden if a preset is selected.
var   sw_ScrollDuration=300;

//if enabled, set step size relative to the scrolled area height by a constant factor.
//thus, if enabled and the visible window/frame is short, so will be the scroll steps.
//when enabled, it overrides gecko internal fixed pixels/line constant. default is true.
// -> this value is overriden if a preset is selected.
var   sw_EnableRelativeStepSize=true;

//if enabled, the step size will vary according to the delay between consecutive wheel events.
//such that fast wheel scroll will cause step size to increase, and vice versa.
//the desired effect is that when you're reading, and want to scroll, the step is small,
//but it you're just browsing through the document, with faster wheel action,
//you'll gain more 'milage' per wheel event. default is true.
// -> this value is overriden if a preset is selected.
var   sw_EnableAdaptiveStep=true;

//if enabled, duration will vary according to the delay between consecutive wheel events,
//such that fast wheel scroll will cause duration to DEcrease, and vice versa.
//the desired effect is to make it slower and smoother while reading, but 'snappy' when browsing.
//default is true.
// -> this value is overriden if a preset is selected.
var   sw_EnableAdaptiveDuration=true;

//if enabled, causes the scroll to 'land gracefully' on the top or bottom edges of the document.
//default is true.
var   sw_EnableSoftEdge=true;

//Limits the frames/sec of the smooth scroll operation.
//for best visual performance, set this value to your monitor refresh rate. default is 100.
//reasonable values are 20 to 120
//note that setting it to high values will make no difference if your cpu isn't up for the task.
//tips: for faster Gecko rendering, set Desktop color to 16bpp, and turn off font-anti-aliasing
//IMPORTANT TIP: if possible, set your monitor refresh rate to 100, and ScrollMaxFPS to either 100 or 50.
//   (other values will cause imprecision due to low resolution internal timers)
var sw_ScrollMaxFPS=100;

//IMPORTANT NOTE ABOUT MODIFIER KEYS:
//by default, mozilla/phoenix/firebird assigns the ctrl key to enlarge/reduce the text size.
//the alt key is assigned to browse history. shift key is not assigned.
//in order for a modifier key to work with this extension, it has to be UNassigned first.
//you can UNassign a modifier key for the mouse wheel as follows:
//mozilla: edit->preferences->advanced->mouse Wheel -> set it to scroll normally.
//phoenix/firebird: about:config in the locationbar bar. then change mousewheel.with<xxx>key.action property to 0

//Select a modifier key for fine small scroll steps. default is 3 (SHIFT key).
//possible values: 0=Disable, 1=ALT, 2=CTRL, 3=SHIFT
var sw_ModifierSmallStep=3;

//Select a modifier key for full page scroll (like PgUp/PgDn). default is 1 (ALT key).
//possible values: 0=Disable, 1=ALT, 2=CTRL, 3=SHIFT
var sw_ModifierBigStep=1;


//Enable/Disable capture supported keyboard events for scroll (Up/Down arrows)
var sw_enableKeyboard=false;


//Select a modifier key for currently disabling smoothwheel. default is 0 (not assigned).
//possible values: 0=Disable, 1=ALT, 2=CTRL, 3=SHIFT
var sw_ModifierDisableSmoothWheel=0;

//curtesy of All-In-One-Gestures author, starting with v0.3, smoothwheel uses a new target detection system
//(that's the system that decides what part of the screen to scroll when the wheel is scrolled)
//the new system is able to scroll inside textareas of forms, drop-down menus and more.
//however, on older Gecko builds, it currently performs less that optimal. So, if you feel that you prefere
//previous version's (v0.2) detection mechanism, set sw_compatibilityDetectionMode to true.
//default is false.
var sw_compatibilityDetectionMode=false;

//********* END OF COMMON CONFIGURATION OPTIONS SECTION ***********

//WHILE THE ABOVE COMMON CONFIGURATIONS ARE MOSTLY ENABLE/DISABLE, THE NEXT
//VALUES CAN BE USED TO FINE-TUNE THE BEHAVIOUR OF EACH FEATURE

//these 2 values control the step size when EnableAdaptiveStep=true.
//they define the factors for the smallest and largest steps, respectively. defaults are .4 and 1.2
//for extreme control (and variation) (requires a steady hand as well), try 0.3 and 3 respectively.
var sw_AdaptiveStepMinStepFactor=.4;
var sw_AdaptiveStepMaxStepFactor=1.2;

//these 2 values control the duration when EnableAdaptiveDuration=true.
//they define the factors for the smallest and largest durations, respectively. defaults are .8 and 2
var sw_AdaptiveDurationMinStepFactor=.8;
var sw_AdaptiveDurationMaxStepFactor=2;

//factor to use when EnableRelativeStepSize is true. default is 0.15 of viewable page height.
var sw_RelativeStepFactor=0.15;

//this factor configures how much "big step" is when ModifierBigStep is used.
//1 will result in normal step. 2 results 2 steps, etc.
//default is 5 of full page height.
var sw_BigStepFactor=5;

//factor for how short the scroll steps are if ModifierSmallStep is used.
//default is 0.1 of normal step.
var sw_ShortStepFactor=0.2;

//#pixels per scroll line. relevant only if EnableRelativeStepSize=false.
//default value is 20 (empirical estimation of internal Gecko value)
var sw_ScrollPx=20;

//the next 4 values control the duration between wheel events that define the
//adaptive behaviour of the adaptiveStep and adaptiveDurations features.

//defaults are .7 and 1.5
var sw_AdaptiveStepMinDurationFactor=.7;
var sw_AdaptiveStepMaxDurationFactor=1.5;

//defaults are .7 and 1.5
var sw_AdaptiveDurationMinDurationFactor=.7;
var sw_AdaptiveDurationMaxDurationFactor=1.5;

//******* it's recommended not to change the next values since it'll prevent optimizations *******/

//tries to make a smooth transition if new scroll event occures while still scrolling. default is true.
// -> this value is overriden if a preset is selected.
// -> changing the default value will prevent code optimizations.
var   sw_EnableSmoothEventTransition=true;

//selects either linear (=0) or various non-linear(=1/2/3) scroll profiles. default is 1.
//possible values:
// 0 : Linear
// 1 : 2nd power (non-linear)
// 2 : 3rd power (non-linear)
// 3 : cosine    (non-linear)
// -> this value is overriden if a preset is selected.
// -> changing the default value will prevent code optimizations.
var   sw_SmoothScrollProfile=1;

//percentage of when to finish transitional profile (from previous velocity to absolute scroll profile)
//default is 1.
// -> this value is overriden if a preset is selected.
// -> changing the default value will prevent code optimizations.
var   sw_TransEnd=1;

//THIS VALUE IS IRRELEVANT WITH THE NEW (v0.3+) DETECTION MECHANISM
//Whether or not to enable WHOLE PAGE scroll when mouse wheel is over form textarea. default is false.
// if set to false, mouse wheel events will NOT be intercepted by this extension while
//   mouse is over a form textarea. instead, the DEFAULT scroll will be issued.
// if set to true, the WHOLE PAGE will scroll smoothly when mouse is over a form textarea.
var sw_EnableWholePageScrollInForms=false;

//controls debug mode on/off. default is false.
var sw_debugMode=false;
//*************** END OF CONFIGURATION SECTION ********************

var sw_LoopInterval=Math.round(1000/sw_ScrollMaxFPS); //fps converted to ms between scroll events

//Global variables (used by more than one function).
var sw_scrollLeft=0; //in pixels
var sw_scrollLeftLastEvent=0; //in pixels
var sw_scrollEventTimeStamp=0;
var sw_scrollDeadline=0;// timestamp
var sw_clientFrame=null;  //holds the object to scroll
var sw_intervalID=null;   //holds the interval timer handle. also indicates whether we're currently scrolling.
var sw_velocityLastEvent=0;
var sw_velocityCurrent=0;
var sw_scrollCurrentStep=1;  //how many pixels to scroll for the current scroll event.
var sw_scrollCurrentDuration=0;  //how many ms to assign for the current scroll event.
var sw_optimizedProfileCodeExists=false; //whether optimized code exist according to various variables
var sw_optimizedHandlerExists=false;//if all vairables match, there's a pre-set optimized handler for default settings.
var sw_maxTop=0;// maximum top to scroll to. =sw_clientFrame.scrollHeight-sw_clientFrame.clientHeight
var sw_isScrollable=false;

//set some of the adaptive functionality parameters only once in advance.
var sw_ASa=0;
var sw_ASb=0;
var sw_ADa=0;
var sw_ADb=0;

function sw_initGlobals(){
  sw_ASa=((sw_AdaptiveStepMinStepFactor-sw_AdaptiveStepMaxStepFactor)/
          (sw_AdaptiveStepMaxDurationFactor-sw_AdaptiveStepMinDurationFactor))/sw_ScrollDuration;//diagonal
  sw_ASb=sw_AdaptiveStepMaxStepFactor-sw_ASa*sw_AdaptiveStepMinDurationFactor; //offset

  sw_ADa=((sw_AdaptiveDurationMaxStepFactor-sw_AdaptiveDurationMinStepFactor)/
          (sw_AdaptiveDurationMaxDurationFactor-sw_AdaptiveDurationMinDurationFactor))/sw_ScrollDuration;//diagonal
  sw_ADb=sw_AdaptiveDurationMinStepFactor-sw_ADa*sw_AdaptiveDurationMinDurationFactor; //offset


  if (sw_intervalID)
    window.clearInterval(sw_intervalID);

  sw_scrollLeft=0;
  sw_scrollLeftLastEvent=0;
  sw_scrollEventTimeStamp=0;
  sw_scrollDeadline=0;
  sw_clientFrame=null;
  sw_intervalID=null;
  sw_velocityLastEvent=0;
  sw_velocityCurrent=0;
  sw_scrollCurrentStep=1;
  sw_scrollCurrentDuration=0;
  sw_maxTop=0;
  sw_isScrollable=false;
  return;
}

function sw_dbgAlert(x){if (sw_debugMode) return alert(x);};
function sw_dbgStringObj(x){return ""+x+"='"+eval(x)+"'\n";};
//if some sanity tests fail for no appearant reason, reset to initial values.
function sw_panicAbort(){
  sw_dbgAlert("SmoothWheel: panic abort:\nPlease post the current page on the smoothwheel forum\n\n"+
    sw_dbgStringObj("sw_scrollLeft")+
    sw_dbgStringObj("sw_scrollLeftLastEvent")+
    sw_dbgStringObj("sw_scrollEventTimeStamp")+
    sw_dbgStringObj("sw_scrollDeadline")+
    sw_dbgStringObj("sw_clientFrame")+
    sw_dbgStringObj("sw_intervalID")+
    sw_dbgStringObj("sw_velocityLastEvent")+
    sw_dbgStringObj("sw_velocityCurrent")+
    sw_dbgStringObj("sw_scrollCurrentStep")+
    sw_dbgStringObj("sw_scrollCurrentDuration")+
    sw_dbgStringObj("sw_optimizedProfileCodeExists")+
    sw_dbgStringObj("sw_optimizedHandlerExists")+
    sw_dbgStringObj("sw_maxTop")+
    sw_dbgStringObj("sw_isScrollable")
  );

  sw_initSmoothWheel();
}

var sw_prefServices = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

function sw_setPref(aPrefType, aPrefString, aValue){
//    alert("set pref ("+aPrefType + ", "+aPrefString+", "+aDefaultValue+")");

    //~ var pref = Components.classes["@mozilla.org/preferences-service;1"]
        //~ .getService(Components.interfaces.nsIPrefBranch);
    try {
      switch (aPrefType) {
        case "bool":
          sw_prefServices.setBoolPref(aPrefString, aValue);
          break;
        case "int":
          sw_prefServices.setIntPref(aPrefString, aValue);
          break;
        case "color":
        case "string":
        case "localizedstring":
        default:
//         pref.setCharPref(aPrefString, aValue);
        sw_prefServices.setCharPref(aPrefString, aValue);
        break;
      }
    }catch(e){
//      if(_DEBUG) dump(e + "\n");
    }
}

const sw_PrefExtensionRoot= "extensions.smoothwheel.";

function sw_getPref(aPrefType, aPrefString, aDefaultValue){
//    alert("get pref ("+aPrefType + ", "+aPrefString+", "+aDefaultValue+")");
//    if(_DEBUG) dump("in getpref\n");
    //~ var pref = Components.classes["@mozilla.org/preferences-service;1"]
        //~ .getService(Components.interfaces.nsIPrefBranch);
    try {
      switch(aPrefType) {
        case "bool":
          return sw_prefServices.getBoolPref(aPrefString);
          //return !aDefaultFlag ? pref.getBoolPref(aPrefString) : pref.getDefaultBoolPref(aPrefString);
        case "int":
          return sw_prefServices.getIntPref(aPrefString);
          //return !aDefaultFlag ? pref.getIntPref(aPrefString) : pref.getDefaultIntPref(aPrefString);
        case "localizedstring":
          return sw_prefServices.getLocalizedUnicharPref(aPrefString); // not working?
        case "color":
        case "string":
        default:
          return sw_prefServices.getCharPref(aPrefString); 
          //return !aDefaultFlag ? pref.copyUnicharPref(aPrefString) : pref.copyDefaultUnicharPref(aPrefString);
      }
    }catch(e) {
       //~ if(_DEBUG) {
        //~ dump("*** no default pref for " + aPrefType + " pref: " + aPrefString + "\n");
        //~ dump(e + "\n");
      //~ }
//      alert ("get pref ("+aPrefString + ", "+aPrefType+") failed");
//      sw_prefServices.deleteBranch(aPrefString);
      sw_setPref(aPrefType, aPrefString, aDefaultValue);
      return aDefaultValue;
    }
    return "!/!ERROR_UNDEFINED_PREF!/!";
}

function sw_readPrefs(){
    //1st, cleanup pref data if doesn't belong to the current smoothwheel version.
    try {
        var prefVersion=sw_prefServices.getCharPref(sw_PrefExtensionRoot+"version");
        if (prefVersion != sw_currentVersion){
            sw_prefServices.deleteBranch(sw_PrefExtensionRoot);
            sw_prefServices.setCharPref(sw_PrefExtensionRoot+"version", sw_currentVersion);
        }
    } catch (e){
        sw_prefServices.deleteBranch(sw_PrefExtensionRoot);
        sw_prefServices.setCharPref(sw_PrefExtensionRoot+"version", sw_currentVersion);
    }
    sw_EnableThisExtension          = !sw_getPref("bool",       sw_PrefExtensionRoot+ "quickDisable", false);
    if (!sw_EnableThisExtension)
        return;
    
    sw_ScrollDuration               = 1*sw_getPref("string",    "extensions.smoothwheel.scrollDuration", "300");

    var css=1*sw_getPref("string",    "extensions.smoothwheel.compoundStepSize", "0.15");
    if (css>0){
        sw_EnableRelativeStepSize=true;
        sw_RelativeStepFactor=css;
    } else {
        sw_EnableRelativeStepSize=false;
        sw_ScrollPx=-css;
    }

    sw_ModifierBigStep              = 1*sw_getPref("string",    sw_PrefExtensionRoot+ "fastScrollModifier", "1");
    sw_ModifierSmallStep            = 1*sw_getPref("string",    sw_PrefExtensionRoot+ "slowScrollModifier", "3");
    sw_compatibilityDetectionMode   = sw_getPref("bool",        sw_PrefExtensionRoot+ "compatibilityMode", false);


    var pas=1*sw_getPref("string",    sw_PrefExtensionRoot+ "presetAdaptiveStep", "2");
    sw_EnableAdaptiveStep=true;
    switch (pas){
        case 0: sw_EnableAdaptiveStep=false; break;
        case 1:sw_AdaptiveStepMinStepFactor=.6;sw_AdaptiveStepMaxStepFactor=1.1;break;
        case 2:sw_AdaptiveStepMinStepFactor=.4;sw_AdaptiveStepMaxStepFactor=1.2;break;
        case 3:sw_AdaptiveStepMinStepFactor=.3;sw_AdaptiveStepMaxStepFactor=1.5;break;
        case 4:sw_AdaptiveStepMinStepFactor=.2;sw_AdaptiveStepMaxStepFactor=2;break;
        default:sw_AdaptiveStepMinStepFactor=.4;sw_AdaptiveStepMaxStepFactor=1.2;
    }

    var pad=1*sw_getPref("string",    sw_PrefExtensionRoot+ "presetAdaptiveDuration", "2");
    sw_EnableAdaptiveDuration=true;
    switch (pad){
        case 0: sw_EnableAdaptiveDuration=false; break;
        case 1: sw_AdaptiveDurationMinStepFactor=.9;sw_AdaptiveDurationMaxStepFactor=1.5;break;
        case 2: sw_AdaptiveDurationMinStepFactor=.8;sw_AdaptiveDurationMaxStepFactor=2;break;
        case 3: sw_AdaptiveDurationMinStepFactor=.5;sw_AdaptiveDurationMaxStepFactor=2.5;break;
        case 4: sw_AdaptiveDurationMinStepFactor=.3;sw_AdaptiveDurationMaxStepFactor=3;break;
        default:sw_AdaptiveDurationMinStepFactor=.8;sw_AdaptiveDurationMaxStepFactor=2;break;
    }

    sw_ScrollMaxFPS                 = 1*sw_getPref("string",    sw_PrefExtensionRoot+ "scrollMaxFPS", "100");
    sw_LoopInterval                 =Math.round(1000/sw_ScrollMaxFPS);    
    sw_EnableSoftEdge               = sw_getPref("bool",        sw_PrefExtensionRoot+ "enableSoftEdge", true);
    sw_ShortStepFactor              = 1*sw_getPref("string",    sw_PrefExtensionRoot+ "shortStepFactor", "0.2");
    sw_BigStepFactor                = 1*sw_getPref("string",    sw_PrefExtensionRoot+ "bigStepFactor", "5");
    sw_enableKeyboard               = sw_getPref("bool",        sw_PrefExtensionRoot+ "enableKeyboard", false);
//    dbg("sw_readPrefs: end");
}


//checks whether the specified modifier key was held when the event was intercepted.
function sw_modifierHeld(e, modifier){
    switch (modifier) {
       case 1: return e.altKey;
       case 2: return e.ctrlKey;
       case 3: return e.shiftKey;
    }
    return false;
}


//avih: improving hack from AIO v0.96: detect new (gecko 1.7 and above cvs trees) browsers
var sw_UAafterRV = window.navigator.userAgent.lastIndexOf("rv:") + 3;
var sw_geckoVer=parseFloat(window.navigator.userAgent.substring(sw_UAafterRV))
var sw_geckoPre_1_7=true;
if (isNaN(sw_geckoVer) || sw_geckoVer>=1.7)
  sw_geckoPre_1_7=false;
//alert("UA:\n"+window.navigator.userAgent+"\ngecko ver: "+sw_geckoVer+"\nis pre1.7: "+sw_geckoPre_1_7);

function sw_findTargetObject(e){
  if (sw_compatibilityDetectionMode)
       return e.target.ownerDocument.defaultView;
  //implicit else. next code is adapted from All-In-One-Gestures v0.96.

  //avih: elsewhere from AIOv0.96
  var contentArea=false, renderingArea=false;
  contentArea = document.getElementById("content");
  try{renderingArea = contentArea.mPanelContainer;} catch (e){};//avih: added: try/catch needed for mail
  const defaultScrollBarSize = 16;

  //avih: AIO 0.96 start
  function sw_findNodeToScroll(initialNode, docBox) {

    function sw_scrollCursorType(neededW, availW, neededH, availH, scrollBarSize) {
      if (neededW <= availW && neededH <= availH) return 3;
      if (neededW > availW && neededH > availH) return 0;
      if (neededW > availW) return ((neededH <= (availH - scrollBarSize)) - 0) << 1;  // 0 or 2
      return (neededW <= (availW - scrollBarSize)) - 0;
    }

    function sw_aioGetStyle(elem, aProp) {
      var p = elem.ownerDocument.defaultView.getComputedStyle(elem, "").getPropertyValue(aProp);
      var val = parseFloat(p);
      if (!isNaN(val)) return Math.ceil(val);
      if (p == "thin") return 1;
      if (p == "medium") return 3;
      if (p == "thick") return 5;
      return 0;
    }

    var retObj = {scrollType: 3, isXML: false, nodeToScroll: null, docWidth: 0,
                  docHeight: 0, clientFrame: null, isBody: false};
    var realWidth, realHeight, bodyEl, docEl, nextNode, currNode;
    docEl = initialNode.ownerDocument.documentElement;
    if (docEl && docEl.nodeName.toLowerCase() == "html") { // walk the tree up looking for something to scroll
       var bodies = docEl.getElementsByTagName("body");
       if (!bodies || !bodies.length) return retObj;
       bodyEl = bodies[0];
       if (initialNode == docEl) nextNode = bodyEl;
//     else if (initialNode.nodeName.toLowerCase() == "select") nextNode = initialNode.parentNode;
       else nextNode = initialNode;
       do
          try {
            currNode = nextNode;
            nextNode = currNode.parentNode;
            if (currNode.clientWidth && currNode.clientHeight &&
                currNode.ownerDocument.defaultView.getComputedStyle(currNode, "").getPropertyValue("overflow") != "hidden") {
                  //avih: hack for FB 0.8+ MOZ 1.6+ etc from AIO v0.96
               if (!sw_geckoPre_1_7 || currNode == bodyEl) {
                  realWidth = currNode.clientWidth + sw_aioGetStyle(currNode, "border-left-width") + sw_aioGetStyle(currNode, "border-right-width");
                  realHeight = currNode.clientHeight + sw_aioGetStyle(currNode, "border-top-width") + sw_aioGetStyle(currNode, "border-bottom-width");
               }
               else {
                  realWidth = currNode.clientWidth;
                  realHeight = currNode.clientHeight;
               }
               retObj.scrollType = sw_scrollCursorType(currNode.scrollWidth, realWidth, currNode.scrollHeight, realHeight, 0);
               if (retObj.scrollType != 3) break;
            }
          }
          catch(err) {}
       while (currNode && currNode != docEl && currNode.nodeName.toLowerCase() != "select");
       retObj.nodeToScroll = currNode;
       retObj.isBody = currNode == bodyEl || currNode == docEl;
       retObj.docWidth = docEl.scrollWidth; retObj.docHeight = docEl.scrollHeight;
    }
    else { // XML document; do our best
       retObj.clientFrame = initialNode.ownerDocument.defaultView;
       retObj.nodeToScroll = initialNode;
       if (docBox) {
          retObj.docWidth = docBox.width; retObj.docHeight = docBox.height;
          try {retObj.scrollType = sw_scrollCursorType(docBox.width, renderingArea.boxObject.width, docBox.height, renderingArea.boxObject.height, defaultScrollBarSize);}catch(e){};
       }
       retObj.isXML = true;
    }
    return retObj;
  }
  //avih: AIO 0.96 end

  var targetDoc = e.target.ownerDocument;
  var insertionNode = (targetDoc.documentElement) ? targetDoc.documentElement : targetDoc;
  var docBox = targetDoc.getBoxObjectFor(insertionNode);
  var scrollObj = sw_findNodeToScroll(e.target, docBox);
  sw_isScrollable=
    (!scrollObj.isXML) &&
    (scrollObj.nodeToScroll.toString().indexOf("XULElement")<0) &&//not an XUL element
    (scrollObj.scrollType<=1); //can be vertically scrolled
  if (sw_isScrollable) {//V scroll needed
//     alert(scrollObj.nodeToScroll+"("+scrollObj.scrollType+")");
     return scrollObj.nodeToScroll;
  }
  return false;
}


function sw_findTargetViewHeight(target){
  if (sw_compatibilityDetectionMode)
     return target.innerHeight;

  //implicit else
  return target.clientHeight;
}


function sw_scrollVert(target, y){
  if (sw_compatibilityDetectionMode)
     return target.scrollBy(0,y);

  //implicit else
  target.scrollTop += y;
}

function sw_contains(a,b){
    return (""+a).indexOf(""+b)>=0;
}

//checks if we're able to scroll the current target according to the event.
//IMPORTANT: with default settings of sw_targetCompatibilityMode==false,
//    sw_findTargetObject(e) MUST be called before for this function to return
//    a valid result.
function sw_isValidTarget(e){
  if (sw_compatibilityDetectionMode){// my old (prior to v0.3) detection code
    var parents= sw_parentTree(e.target);
    var tmpClientFrame= sw_findTargetObject(e);//e.target.ownerDocument.defaultView;
    if (
          tmpClientFrame != "[object Window]" ||
          parents.indexOf("HTMLTextAreaElement")>=0  && (!sw_EnableWholePageScrollInForms) ||
          parents.indexOf("XULElement")>=0 ||
          parents.indexOf("HTMLEmbedElement")>=0 ||
          //sw_parents.indexOf("HTMLImageElement")>=0 ||
          parents.indexOf("HTMLOptionElement")>=0
       )
       return false; //default handler will take it from here.
    else
    return true;
  }

  if (sw_enableKeyboard && e.type=="keypress"){
    var dm=false;
    try{
        dm=(e.target.ownerDocument.designMode=="on");
    }catch(e){};
    if (0
        || sw_contains(e.target, "Option")
        || sw_contains(e.target, "Select")
        || sw_contains(e.target, "TextArea")
        || sw_contains(e.target, "Embed")
        || dm
        ){
        return false;
    }
  }
  
  //implicit else

  return sw_isScrollable;
}


//called when wheel event captured, while NOT scrolling.
//determines whether to leave it alone, or to use it (and initilize the vars)
function sw_acquireEvent(e){
//    sw_initSmoothWheel();
    var tmpClientFrame= sw_findTargetObject(e);
    if ((sw_modifierHeld(e, sw_ModifierDisableSmoothWheel)) || (!sw_isValidTarget(e)))
       return false;

    //implicit else: ok. intercepting event. initialize variables, step size etc.
    sw_intervalID= window.setInterval(sw_smoothWheelLoop,sw_LoopInterval);
    sw_clientFrame= tmpClientFrame;

    var viewHeight= sw_findTargetViewHeight(sw_clientFrame);//clientFrame.innerHeight;
    sw_scrollCurrentStep=(sw_EnableRelativeStepSize)? viewHeight*sw_RelativeStepFactor: sw_ScrollPx;//sw_ScrollPx is now absolute. orig: *(Math.abs(e.detail));

    if (sw_modifierHeld(e, sw_ModifierBigStep))
       sw_scrollCurrentStep=sw_scrollCurrentStep*sw_BigStepFactor;
    if (sw_modifierHeld(e, sw_ModifierSmallStep))
       sw_scrollCurrentStep= sw_scrollCurrentStep*sw_ShortStepFactor;

    sw_maxTop=sw_clientFrame.scrollHeight-sw_clientFrame.clientHeight;
//    alert("OrigHeight="+viewHeight+"\nMaxTop="+sw_maxTop);

    sw_velocityCurrent=0;
    return true;
}

var sw_lastWheelLoopEvent=0;
var sw_dtn_0=0;
var sw_dtn_1=0;
var sw_dtn_2=0;

function sw_clipValue(value, minValue, maxValue){
  return (value>maxValue)?maxValue:((value<minValue)?minValue:value);
}


//var sw_dbgIsInitialized=false;
//SmoothWheel wheel event handler, registered for handling DOMMouseScroll events
function smoothWheelHandler(e){
//       if (!sw_dbgIsInitialized){
//            sw_debugMode=true;
//            sw_addMonitorPanel(e);
//            sw_addMonitorItem("velocity",0,2);
//            sw_addMonitorItem("v.last",0,2);
//            sw_addMonitorItem("perc",0,100);
//            sw_addMonitorItem("dur",sw_ScrollDuration*sw_AdaptiveDurationMinStepFactor,sw_ScrollDuration*sw_AdaptiveDurationMaxStepFactor);
//            sw_addMonitorItem("durAvg",sw_ScrollDuration*sw_AdaptiveDurationMinStepFactor,sw_ScrollDuration*sw_AdaptiveDurationMaxStepFactor);
//            sw_dbgIsInitialized=true;
//       }
    var alreadyScrolling=(sw_velocityCurrent!=0);
    if (!alreadyScrolling){
        sw_readPrefs();
        if  (!sw_EnableThisExtension){
            return;
        }
    }
     

    if (e.type=="keypress"){
        if (!sw_enableKeyboard || e.keyCode!=e.DOM_VK_DOWN && e.keyCode!=e.DOM_VK_UP)
            return;

        e.sw_detail=(e.keyCode==e.DOM_VK_DOWN)?1:-1
    } else {
        e.sw_detail=e.detail;
    }
      

      var now= (new Date()).getTime();
      if (!alreadyScrolling){//already scrolling, do average 2
        sw_dtn_1=
        sw_dtn_2=
          sw_ScrollDuration*sw_AdaptiveDurationMaxStepFactor;
      } else {
        sw_dtn_2=sw_dtn_1;
        sw_dtn_1=sw_dtn_0;
      }
      sw_dtn_0=(now-sw_scrollEventTimeStamp);
      var dtn=(sw_dtn_0+sw_dtn_1+sw_dtn_2)/3; //average last 3 event deltas.
/*
      if (sw_optimizedHandlerExists){// if all settings match, use a pre-configured handler.
        if (!sw_intervalID) if (!sw_acquireEvent(e))  return;

        var sf=sw_ASa*dtn+sw_ASb, df=sw_ADa*dtn+sw_ADb;
        sw_scrollCurrentDuration=300*((df<.8)?.8:((df>2)?2:df));
        sw_scrollLeft += sw_scrollCurrentStep*((e.sw_detail>0)?1:-1)*((sf>1.2)?1.2:((sf<.4)?.4:sf));
        var st=sw_clientFrame.scrollTop, sd=st+sw_scrollLeft;
        sw_scrollLeft=(sd<0)?-st:((sd>sw_maxTop)?sw_maxTop-st:sw_scrollLeft);
        sw_scrollEventTimeStamp=sw_velocityCurrent?sw_lastWheelLoopEvent:now;//now;
        sw_scrollDeadline= sw_scrollEventTimeStamp + sw_scrollCurrentDuration;
        sw_scrollLeftLastEvent= sw_scrollLeft;
        sw_velocityLastEvent= sw_velocityCurrent;
        e.preventDefault();
        e.stopPropagation();
        return;
      }
*/

      //implicit else, consider all variables explicitly
      if (!sw_intervalID) //if not currently scrolling
        if (!sw_acquireEvent(e)) //if event should not be intercepted
          return;  //let default handler kick in
        //implicit else: event acquired and vars initialized
      //implicit else: currently scrolling -> modify target and deadline.

      e.preventDefault();
      e.stopPropagation();

      var suggestedStepFactor=sw_clipValue(sw_ASa*dtn+sw_ASb, sw_AdaptiveStepMinStepFactor, sw_AdaptiveStepMaxStepFactor);
      var suggestedDurationFactor=sw_clipValue(sw_ADa*dtn+sw_ADb, sw_AdaptiveDurationMinStepFactor, sw_AdaptiveDurationMaxStepFactor);
      sw_scrollCurrentDuration=sw_ScrollDuration*((sw_EnableAdaptiveDuration)?suggestedDurationFactor:1);
      sw_scrollLeft += ((e.sw_detail>0)?1:-1)*sw_scrollCurrentStep*((sw_EnableAdaptiveStep)?suggestedStepFactor:1);

      if (sw_EnableSoftEdge){
         var destination=sw_clientFrame.scrollTop+sw_scrollLeft;
         if (destination < 0) sw_scrollLeft = -sw_clientFrame.scrollTop;
         else if (destination > sw_clientFrame.scrollHeight-sw_clientFrame.clientHeight)
            sw_scrollLeft = sw_clientFrame.scrollHeight-sw_clientFrame.clientHeight - sw_clientFrame.scrollTop
      }

      sw_scrollEventTimeStamp=alreadyScrolling?sw_lastWheelLoopEvent:now;
      sw_scrollDeadline= sw_scrollEventTimeStamp + sw_scrollCurrentDuration;
      sw_scrollLeftLastEvent= sw_scrollLeft;
      sw_velocityLastEvent= sw_velocityCurrent;
}

//loop for smooth scrolling the page
function sw_smoothWheelLoop(){
      var toScroll, currentTime, currentPercentage, timeLeft;
      currentTime=(new Date()).getTime();
      if (sw_lastWheelLoopEvent==currentTime) {//we're back too soon. means previous loop was delayed.
           window.clearInterval(sw_intervalID);
           sw_intervalID= window.setInterval(sw_smoothWheelLoop,sw_LoopInterval);
           return;
      }
      sw_lastWheelLoopEvent= currentTime;
      if (//saniti tests
        isNaN(sw_scrollDeadline) ||
        isNaN(sw_scrollCurrentDuration) ||
        isNaN(sw_scrollLeftLastEvent) ||
        isNaN(sw_velocityCurrent) ||
        isNaN(sw_scrollLeft) ||
        !sw_clientFrame
      ){
        sw_panicAbort();
      }

      timeLeft= sw_scrollDeadline - currentTime;

      if (timeLeft<=0)  //we're past the deadline. scroll what's left.
          toScroll=sw_scrollLeft;
      else{ //calculate next scroll destination according to scroll profile
          currentPercentage= 1 - timeLeft/sw_scrollCurrentDuration;
          toScroll= Math.round (sw_scrollLeftLastEvent*(sw_scrollProfile(currentPercentage) - 1) + sw_scrollLeft);
          if ((sw_scrollLeft-toScroll)*sw_scrollLeft < 0) //if going past destination point
              toScroll=sw_scrollLeft;
      }

//       sw_setMonitorItemValue("velocity",sw_velocityCurrent);
//       sw_setMonitorItemValue("v.last",sw_velocityLastEvent);
//       sw_setMonitorItemValue("perc",100*currentPercentage);
//       sw_setMonitorItemValue("durAvg",sw_scrollCurrentDuration);
//       sw_setMonitorItemValue("dur", sw_ScrollDuration*sw_clipValue(sw_ADa*sw_dtn_0+sw_ADb, sw_AdaptiveDurationMinStepFactor, sw_AdaptiveDurationMaxStepFactor));
//       sw_renderMonitor();
      if ((toScroll==0) && (sw_scrollLeft!=0))
        return;
      sw_scrollVert(sw_clientFrame, toScroll);
      sw_scrollLeft -= toScroll;

      if (sw_scrollLeft==0){//no more scroll needed
         window.clearInterval(sw_intervalID);
      	 sw_intervalID=null;
         sw_velocityCurrent=0;
         return;
      }
}


//vars for optimized profile only.
var sw_percentage, sw_p1, sw_p2, sw_p21, sw_p2v;

//transform percentage of time to percentage of location according to a scroll profile
//it might consider an implicit parameter of the speed of the scroll at the time the event was intercepted.
function sw_scrollProfile(percentage){
        if (sw_optimizedProfileCodeExists){
             sw_p=percentage; sw_p1=1-sw_p; sw_p2=sw_p1*sw_p1; sw_p21=1-sw_p2; sw_p2v=sw_p2*sw_velocityLastEvent;
             sw_velocityCurrent= sw_p2v + sw_p21*2*sw_p1;
             return sw_p2v*sw_p + sw_p21*sw_p21; //location
        }
        // the above optimized code (hardcoded values, inlined functioned) is used if:
        //    SmoothScrollProfile=1 (->profilePower2) and EnableSmoothEventTransition=true and TransEnd=1
        // the optimized code does the stuff instead of running fully (sometimes multiple times) through the next functions:
        //    scrollProfile(p), transition(p,x,y),
        //    profilePower2(p), profilePower2Velocity(p),
        //    profileConstVelocity(p), profileConstVelocityVelocity(p)
        // the best optimization though would be to get 100-200 precalculated results into a table, and merge the value with velocityLastEvent in runtime;)
        //optimized transition(p,x,y)=p2*x+p21*y //p2 and p21 are already functions of p

        //implicit else: non-optimized path, consider all settings explicitly.
        var location, velocity;
        switch (sw_SmoothScrollProfile){
               case 0:  location= sw_profileLinear(percentage); break;
               case 1:  location= sw_profilePower2(percentage); break;
               case 2:  location= sw_profilePower3(percentage); break;
               case 3:  location= sw_profileCosine(percentage); break;
               default: location= sw_profileLinear(percentage); break;
        }
        if (sw_EnableSmoothEventTransition){
           switch (sw_SmoothScrollProfile){
               case 0:  velocity= sw_profileLinearVelocity(percentage); break;
               case 1:  velocity= sw_profilePower2Velocity(percentage); break;
               case 2:  velocity= sw_profilePower3Velocity(percentage); break;
               case 3:  velocity= sw_profileCosineVelocity(percentage); break;
               default: velocity= sw_profileLinearVelocity(percentage); break;
           }
           location= sw_transition(percentage, sw_profileConstVelocity(percentage), location);
           sw_velocityCurrent= sw_transition(percentage, sw_profileConstVelocityVelocity(percentage), velocity);
        }
        return location;
}


//returns a weighted average of 2 profiles according (non-linear) to percentage.
function sw_transition(percentage, srcProfile, destProfile){
         var srcPer= ((sw_TransEnd-percentage)/sw_TransEnd)*((sw_TransEnd-percentage)/sw_TransEnd);
         var destPer= 1-srcPer;
         return (percentage < sw_TransEnd)?
                        srcPer*srcProfile + destPer*destProfile: destProfile;
}


//various scroll profiles and their velocities (derivatives) functions.
function sw_profileLinear(percentage){return percentage;}
function sw_profileLinearVelocity(percentage){return 1;}

function sw_profilePower2(percentage){return 1-(1-percentage)*(1-percentage);}
function sw_profilePower2Velocity(percentage){return 2*(1-percentage);}

function sw_profilePower3(percentage){return 1-(1-percentage)*(1-percentage)*(1-percentage);}
function sw_profilePower3Velocity(percentage){return 3*(1-percentage)*(1-percentage);}

function sw_profileCosine(percentage){return .5-.5*Math.cos(Math.PI*percentage);}
function sw_profileCosineVelocity(percentage){return .5*Math.sin(Math.PI*percentage)*Math.PI;}

//using implicit parameter velocityLastEvent.
//this profile does NOT achieve destination point on deadline.
//it represents a linear motion at the speed that was recorded when the event was intercepted.
//this profile is used for transition when a scroll event is intercepted while still scrolling.
function sw_profileConstVelocity(percentage){return sw_velocityLastEvent*percentage;}
function sw_profileConstVelocityVelocity(percentage){return sw_velocityLastEvent;}


// start: functions used by old target detection code
function sw__generalParentTree(node, parentStringElement, counter){
  if (!node)
     return "("+counter+") [object null("+parentStringElement+")]";

  return ""+sw__generalParentTree(eval("node."+parentStringElement), parentStringElement, counter+1)+ "," + node;
}

function sw_generalParentTree(baseNode, selectionStr, parentElementStr){
         return selectionStr+": "+sw__generalParentTree(eval("baseNode."+selectionStr), parentElementStr, 0);
}

//return a string of all parentNode elements (recursively) for the input node
function sw_parentTree(node){
  if (!node)
     return "[object null(generatedString)]";

  return ""+sw_parentTree(node.parentNode)+ "," + node;
}
// end: functions used by old target detection code


//initialize variable according to the preset, and determine whether optimization is possible.
function sw_initSmoothWheel(){
    sw_readPrefs();

    if (sw_PresetSettings >= 1 && sw_PresetSettings <=11){
    //[ScrollDuration, EnableRelativeStepSize, EnableAdaptiveStep, EnableAdaptiveDuration, SmoothScrollProfile, EnableSmoothEventTransition, TransEnd]
        var presets=[, 
                       [100,  true,  true , true ,  1, true,  1], //1
                       [200,  true,  true , true ,  1, true,  1], //2
                       [500,  true,  false, false,  1, true,  1], //3 : was default in v0.2
                       [1000, true,  false, false,  1, true,  1], //4
                       [300,  true,  true,  false,  1, true,  1], //5
                       [300,  true,  false, true ,  1, true,  1], //6
                       [300,  true,  true , true ,  1, true,  1], //7 : default with v0.3, v0.4
                       [500,  true,  true , false , 1, true,  1], //8
                       [500,  true,  true , true ,  1, true,  1], //9
                       [250,  false, false, false,  0, false, 1], //10
                       [150,  false, false, false,  2, false, 1]  //11
                       [150,  true,  false, false,  1, true,  1], //12
                       [300,  true,  false, false,  1, true,  1], //13
                     ];
       sw_ScrollDuration=              presets[sw_PresetSettings][0];
       sw_EnableRelativeStepSize=      presets[sw_PresetSettings][1];
       sw_EnableAdaptiveStep=          presets[sw_PresetSettings][2];
       sw_EnableAdaptiveDuration=      presets[sw_PresetSettings][3];
       sw_SmoothScrollProfile=         presets[sw_PresetSettings][4];
       sw_EnableSmoothEventTransition= presets[sw_PresetSettings][5];
       sw_TransEnd=                    presets[sw_PresetSettings][6];
    }

    sw_initGlobals();
    sw_optimizedProfileCodeExists= false;//(sw_SmoothScrollProfile==1) && (sw_EnableSmoothEventTransition==true) &&  (sw_TransEnd==1);

    sw_optimizedHandlerExists = false; /*(
        sw_EnableThisExtension &&
        sw_EnableAdaptiveStep &&
        sw_EnableAdaptiveDuration &&
        sw_EnableSoftEdge &&
        (sw_ScrollDuration==300) &&
        (sw_AdaptiveStepMinStepFactor==0.4) &&
        (sw_AdaptiveStepMaxStepFactor==1.2) &&
        (sw_AdaptiveDurationMinStepFactor==.8) &&
        (sw_AdaptiveDurationMaxStepFactor==2)
    );
    */
}

//runs once, when the file is loaded.
sw_initSmoothWheel();


// ********** start: debug/test functions *************

// display a debug text on the javascript console.
function dbg(t){
//         that's the best way i found so far. i assume there's a better one though.
    try{
        window.setTimeout("DBG: -->" +t+ "<--",10);
    } catch(e){};
}

// ********** end: debug/test functions *************



/***************************************************************************
SmoothWheel extension for Mozilla/Phoenix/Firebird/Netscape
SmoothWheel home page: http://smoothwheel.mozdev.org
Download latest version: http://downloads.mozdev.org/smoothwheel/smoothwheel.xpi
Licence: GPL 2.0
Copyright 2003 Avi Halachmi (avihpit (*at*) yahoo (*dot*) com)
Current version: v0.44.10.20071026 (2007-10-26)

About:
The smoothwheel extension will scroll the page smoothly with the mouse wheel.
SmoothWheel utilizes several smart algorithms for smoothness and adaptive behaviour.
It can enhance/replace the internal Gecko smoothscroll which was introduced on 2003-03-24.
Tested applications: Mozilla/Firebird/Thunderbird/Phoenix/Netscape.
Tested platforms: win32, Linux, FreeBsd/i386, Mac OS/X (theoretically should work on all platforms).

Features:
- Can be disabled easily, even in Mozilla, Netscape or Thunderbird.
- Automatically utilizes strong CPUs for smoother scroll.
- Very configurable, including 10 presets.
- Scroll steps relative to scrollable area height (configurable).
- Advanced Optimized algorithm for a VERY smooth scroll in many scenarios.
- Soft-Edge : Lands gracefully on top/bottom of document.
- Adaptive-Step : Shorter duration between wheel events -> more pixels/event.
- Adaptive-Duration : Shorter duration between wheel events -> scrolls faster.
- Modifier keys for small/full page steps (SHIFT and ALT respectively by default).
- Modifier key for temporarily disabling smoothwheel (none by default).
- Timer based operation: Always 'finishes on time'.
- Lets the default mouse wheel handler work if smoothwheel can't handle it.

Todo:
- Make it work on XUL panes (mail-and-news top/left panes, irc left/bottom panes, etc).
- Possibly improve installation.

/////////////////////////////////////////////////////////////////////////////
changelog:
/////////////////////////////////////////////////////////////////////////////
v0.44.5.20050523 (2005-05-23)
- added more variables to the configuration panel
- modified prefutils a bit to allow foating point values (as strings)
- more options exposed, better usability by rearrangement of settings and about panel.
- clean preferences when upgrading a version
/////////////////////////////////////////////////////////////////////////////

v0.43.1.20041107 (2004-11-07)
- max-version reduced such that it's accepted by update.mozilla.org
- added build date to the version number
- added tooltip for the modifiers section in the configuration panel

/////////////////////////////////////////////////////////////////////////////
v0.43 (2004-11-07)

improvements (v0.43):
- Added configuration dialog for Firefox/Thunderbird  (from the extensions panel)
  - in mozilla/netscape, enter address: chrome://smoothwheel/content/prefoverlay.xul

/////////////////////////////////////////////////////////////////////////////
v0.42 (2004-10-22)

improvements (v0.42):
- Added support for Firefox-1.0 and Thunderbird-1.0
  - max supported versions: FF: 5.0, TB: 5.0 (I've had it with max-version)

/////////////////////////////////////////////////////////////////////////////
v0.41 (2004-07-03)

improvements (v0.41):
- Modified to also support Firefox-0.9 and Thunderbird-0.7 new Extension-Manager API
  - max supported versions: FF: 0.999, TB: 0.799 (due to possible more EM API changes)

/////////////////////////////////////////////////////////////////////////////
v0.4 (2004-02-04)

bugFix (v0.4):
- Refined algorithms implementation (a bit more responsive)
- Fixed: occasional misbehaviour if not using a preset.

improvements (v0.4):
- Averaged mouse wheel events (better predictability and stability)

/////////////////////////////////////////////////////////////////////////////
v0.33.1 (2004-02-01)

bugFix (v0.33.1):
- Removed 'handled(normal)' alert if default settings are changed.

/////////////////////////////////////////////////////////////////////////////
v0.33 (2004-01-27)

bugFix (v0.33):
- Improved compatibility with latest Moz abd Birds releases (detection code from AIO v0.96)

/////////////////////////////////////////////////////////////////////////////
v0.32 (2003-08-02)

bugFix (v0.32):
- Works in Mozilla-Thunderbird and Help window (Mozilla/Netscape).
- Improved compatibility with Mozilla-Firebird 0.6.1 and Mozilla-Thunderbird 0.1.

/////////////////////////////////////////////////////////////////////////////
v0.31 (2003-06-05)

bugFix (v0.31):
- Fixed a compatibility issue with Scroll-Wheel-Navigation at All-In-One-Gestures extension.

/////////////////////////////////////////////////////////////////////////////
v0.3 (22-May-2003)

bugFix (v0.3):
- new target detection code, courtesy of All-In-One-Gestures author.
- cleanup of namespace (precaution measures)

new features (v0.3):
- now working on view source (all browsers) and composer.
- modifier key for temporarily disabling SmoothWheel.
- Soft-Edge : Lands gracefully on top/bottom of document.
- Adaptive-Step : Shorter duration between wheel events -> more pixels/event.
- Adaptive-Duration : Shorter duration between wheel events -> scrolls faster.
- more optimizations for event handler if default settings are kept.
- some more presets to reflect the new features.

status (v0.3):

scrolling SMOOTHLY on:
- mozilla: browser, forms, view-source, mail-and-news (message windows), composer, irc (main window)
- phoenix/firebird: browser, forms, view-source

letting default scroll handler do it's stuff on:
- mozilla: top/left panes of mail-and-news, left/bottom irc panes, all XUL panes, about:config
- phoenix/firebird: about:config, all XUL panes

breaking wheel scroll functionality on (v0.2):
- couldn't find any so far.

/////////////////////////////////////////////////////////////////////////////
v0.2 (15-Apr-2003)

bugFix (v0.2):
- much better target detection code. will let default scroll handler do it's stuff if we can't handle properly.
- better timing of scroll loop
- fps limit (for fast CPUs)

new features (v0.2):
- modifier keys for slow/fullPage scroll
- improved algorithm: smooth transition if scroll event occures while still scrolling
- much more configuration options (including 6 presets)
- optimized code for the most common settings


status (v0.2):

scrolling SMOOTHLY on:
- mozilla: browser, view-source, irc (main window), mail-and-news (message windows)
- phoenix: browser

letting default scroll handler do it's stuff on:
- mozilla: browser forms, some mail-and-news panes, some irc panes,
           composer, all XUL panes, about:config, more..
- phoenix: browser forms, view-source, about:config, all XUL panes, more..

breaking wheel scroll functionality on (v0.2):
- couldn't find any so far.

/////////////////////////////////////////////////////////////////////////////
v0.1 (22-Mar-2003)

status (v0.1):

scrolling SMOOTHLY on:
- mozilla: browser, view-source, irc (main window), mail-and-news (message windows)
- phoenix: browser

letting default scroll handler do it's stuff on:
- mozilla: composer
- phoenix: view-source

breaking wheel scroll functionality on:
- mozilla: browser forms, some mail-and-news panes, some irc panes, all XUL panes, more...
- phoenix: browser forms, all XUL panes, more...
***************************************************************************
*/
